@@ -1,6 +1,7 @@
 /*
     GNOME Commander - A GNOME based file manager
     Copyright (C) 2001-2006 Marcus Bjurman
+    Copyright (C) 2007-2010 Piotr Eljasiak
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,6 +19,7 @@
 */
 
 #include <config.h>
+
 #include "gnome-cmd-includes.h"
 #include "gnome-cmd-file-popmenu.h"
 #include "gnome-cmd-file.h"
@@ -26,12 +28,10 @@
 #include "gnome-cmd-main-win.h"
 #include "plugin_manager.h"
 #include "gnome-cmd-python-plugin.h"
+#include "gnome-cmd-user-actions.h"
 #include "utils.h"
 #include "cap.h"
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif
 #include <fnmatch.h>
 
 using namespace std;
@@ -50,7 +50,8 @@ typedef struct
 
 static GtkMenuClass *parent_class = NULL;
 
-struct _GnomeCmdFilePopmenuPrivate
+
+struct GnomeCmdFilePopmenuPrivate
 {
     GList *data_list;
 };
@@ -90,11 +91,13 @@ static void cb_exec_default (GtkMenuItem *menu_item, GList *files)
 
     for (; files; files = files->next)
     {
-        GnomeCmdFile *finfo = (GnomeCmdFile *) files->data;
-        GnomeVFSMimeApplication *vfs_app = gnome_vfs_mime_get_default_application (finfo->info->mime_type);
+        GnomeCmdFile *f = (GnomeCmdFile *) files->data;
+        GnomeVFSMimeApplication *vfs_app = gnome_cmd_app_get_default_application (f);
+
         if (vfs_app)
         {
             OpenWithData *data = (OpenWithData *) g_hash_table_lookup (hash, vfs_app->id);
+
             if (!data)
             {
                 data = g_new0 (OpenWithData, 1);
@@ -104,19 +107,19 @@ static void cb_exec_default (GtkMenuItem *menu_item, GList *files)
             }
 
             gnome_vfs_mime_application_free (vfs_app);
-            data->files = g_list_append (data->files, finfo);
+            data->files = g_list_append (data->files, f);
         }
+        else
+            gnome_cmd_show_message (NULL, f->info->name, _("Couldn't retrieve MIME type of the file."));
     }
 
-    g_hash_table_foreach (hash, (GHFunc)htcb_exec_with_app, NULL);
+    g_hash_table_foreach (hash, (GHFunc) htcb_exec_with_app, NULL);
+
     g_hash_table_destroy (hash);
 }
 
 
-static gboolean
-on_open_with_other_ok (GnomeCmdStringDialog *string_dialog,
-                       const gchar **values,
-                       GList *files)
+static gboolean on_open_with_other_ok (GnomeCmdStringDialog *string_dialog, const gchar **values, GList *files)
 {
     GtkWidget *term_check = lookup_widget (GTK_WIDGET (string_dialog), "term_check");
 
@@ -126,21 +129,19 @@ on_open_with_other_ok (GnomeCmdStringDialog *string_dialog,
         return FALSE;
     }
 
-    gchar *cmd = g_strdup_printf ("%s ", values[0]);
+    string cmd = values[0];
 
     for (; files; files = files->next)
     {
-        gchar *path = gnome_cmd_file_get_real_path (GNOME_CMD_FILE (files->data));
-        gchar *tmp = cmd;
-        gchar *arg = g_shell_quote (path);
-        cmd = g_strdup_printf ("%s %s", tmp, arg);
-        g_free (arg);
-        g_free (path);
-        g_free (tmp);
+        cmd += ' ';
+        cmd += stringify (gnome_cmd_file_get_quoted_real_path (GNOME_CMD_FILE (files->data)));
     }
 
-    run_command (cmd, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (term_check)));
-    g_free (cmd);
+    GnomeCmdFileSelector *fs = gnome_cmd_main_win_get_fs (main_win, ACTIVE);
+    GnomeCmdDir *dir = fs->get_directory();
+    gchar *dpath = gnome_cmd_file_get_real_path (GNOME_CMD_FILE (dir));
+    run_command_indir (cmd.c_str(), dpath, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (term_check)));
+    g_free (dpath);
 
     return TRUE;
 }
@@ -152,14 +153,14 @@ static void on_open_with_other (GtkMenuItem *menu_item, GList *files)
     GtkWidget *dialog;
 
     dialog = gnome_cmd_string_dialog_new (_("Open with other..."), labels, 1,
-                                          (GnomeCmdStringDialogCallback)on_open_with_other_ok, files);
+                                          (GnomeCmdStringDialogCallback) on_open_with_other_ok, files);
 
     g_return_if_fail (GNOME_CMD_IS_STRING_DIALOG (dialog));
 
     GtkWidget *term_check = create_check (dialog, _("Needs terminal"), "term_check");
 
     gtk_widget_ref (dialog);
-    gtk_object_set_data_full (GTK_OBJECT (menu_item), "new_textfile_dialog", dialog, (GtkDestroyNotify)gtk_widget_unref);
+    gtk_object_set_data_full (GTK_OBJECT (menu_item), "new_textfile_dialog", dialog, (GtkDestroyNotify) gtk_widget_unref);
 
     gnome_cmd_dialog_add_category (GNOME_CMD_DIALOG (dialog), term_check);
 
@@ -169,9 +170,9 @@ static void on_open_with_other (GtkMenuItem *menu_item, GList *files)
 
 static void on_execute (GtkMenuItem *menu_item, GList *files)
 {
-    GnomeCmdFile *finfo = GNOME_CMD_FILE (files->data);
+    GnomeCmdFile *f = GNOME_CMD_FILE (files->data);
 
-    gnome_cmd_file_execute (finfo);
+    gnome_cmd_file_execute (f);
 }
 
 
@@ -254,8 +255,8 @@ inline gboolean fav_app_matches_files (GnomeCmdApp *app, GList *files)
         case APP_TARGET_ALL_DIRS:
             for (; files; files = files->next)
             {
-                GnomeCmdFile *finfo = (GnomeCmdFile *) files->data;
-                if (finfo->info->type != GNOME_VFS_FILE_TYPE_DIRECTORY)
+                GnomeCmdFile *f = (GnomeCmdFile *) files->data;
+                if (f->info->type != GNOME_VFS_FILE_TYPE_DIRECTORY)
                     return FALSE;
             }
             return TRUE;
@@ -263,8 +264,8 @@ inline gboolean fav_app_matches_files (GnomeCmdApp *app, GList *files)
         case APP_TARGET_ALL_FILES:
             for (; files; files = files->next)
             {
-                GnomeCmdFile *finfo = (GnomeCmdFile *) files->data;
-                if (finfo->info->type != GNOME_VFS_FILE_TYPE_REGULAR)
+                GnomeCmdFile *f = (GnomeCmdFile *) files->data;
+                if (f->info->type != GNOME_VFS_FILE_TYPE_REGULAR)
                     return FALSE;
             }
             return TRUE;
@@ -272,9 +273,9 @@ inline gboolean fav_app_matches_files (GnomeCmdApp *app, GList *files)
         case APP_TARGET_ALL_DIRS_AND_FILES:
             for (; files; files = files->next)
             {
-                GnomeCmdFile *finfo = (GnomeCmdFile *) files->data;
-                if (finfo->info->type != GNOME_VFS_FILE_TYPE_REGULAR
-                    && finfo->info->type != GNOME_VFS_FILE_TYPE_DIRECTORY)
+                GnomeCmdFile *f = (GnomeCmdFile *) files->data;
+                if (f->info->type != GNOME_VFS_FILE_TYPE_REGULAR
+                    && f->info->type != GNOME_VFS_FILE_TYPE_DIRECTORY)
                     return FALSE;
             }
             return TRUE;
@@ -283,10 +284,14 @@ inline gboolean fav_app_matches_files (GnomeCmdApp *app, GList *files)
             for (; files; files = files->next)
             {
                 gboolean ok = FALSE;
+#ifdef FNM_CASEFOLD
                 gint fn_flags = FNM_NOESCAPE | FNM_CASEFOLD;
+#else
+                gint fn_flags = FNM_NOESCAPE;
+#endif
 
-                GnomeCmdFile *finfo = (GnomeCmdFile *) files->data;
-                if (finfo->info->type != GNOME_VFS_FILE_TYPE_REGULAR)
+                GnomeCmdFile *f = (GnomeCmdFile *) files->data;
+                if (f->info->type != GNOME_VFS_FILE_TYPE_REGULAR)
                     return FALSE;
 
                 // Check that the file matches at least one pattern
@@ -294,7 +299,7 @@ inline gboolean fav_app_matches_files (GnomeCmdApp *app, GList *files)
                 for (; patterns; patterns = patterns->next)
                 {
                     gchar *pattern = (gchar *) patterns->data;
-                    ok |= fnmatch (pattern, finfo->info->name, fn_flags) == 0;
+                    ok |= fnmatch (pattern, f->info->name, fn_flags) == 0;
                 }
 
                 if (!ok) return FALSE;
@@ -324,7 +329,7 @@ static void destroy (GtkObject *object)
 {
     GnomeCmdFilePopmenu *menu = GNOME_CMD_FILE_POPMENU (object);
 
-    g_list_foreach (menu->priv->data_list, (GFunc)g_free, NULL);
+    g_list_foreach (menu->priv->data_list, (GFunc) g_free, NULL);
     g_list_free (menu->priv->data_list);
 
     g_free (menu->priv);
@@ -388,14 +393,14 @@ inline gchar *string_double_underscores (const gchar *string)
 }
 
 
-inline gchar *get_default_application_action_name(GList *files)
+inline gchar *get_default_application_action_name (GList *files)
 {
     if (g_list_length(files)>1)
         return g_strdup(_("_Open"));
 
-    GnomeCmdFile *finfo = (GnomeCmdFile *) files->data;
-    gchar *uri_str = gnome_cmd_file_get_uri_str (finfo);
-    GnomeVFSMimeApplication *app = gnome_vfs_mime_get_default_application_for_uri (uri_str, finfo->info->mime_type);
+    GnomeCmdFile *f = (GnomeCmdFile *) files->data;
+    gchar *uri_str = gnome_cmd_file_get_uri_str (f);
+    GnomeVFSMimeApplication *app = gnome_vfs_mime_get_default_application_for_uri (uri_str, f->info->mime_type);
 
     g_free (uri_str);
 
@@ -444,29 +449,33 @@ GtkWidget *gnome_cmd_file_popmenu_new (GnomeCmdFileList *fl)
 
     static GnomeUIInfo other_uiinfo[] =
     {
-        GNOMEUIINFO_ITEM_STOCK(N_("Cu_t"), NULL, on_cut, GNOME_STOCK_MENU_CUT),
-        GNOMEUIINFO_ITEM_STOCK(N_("_Copy"), NULL, on_copy, GNOME_STOCK_MENU_COPY),
-        GNOMEUIINFO_ITEM_STOCK(N_("_Delete"), NULL, on_delete, GNOME_STOCK_MENU_TRASH),
-        GNOMEUIINFO_ITEM_NONE (N_("Rename"), NULL, on_rename),
+        GNOMEUIINFO_ITEM_STOCK(N_("Cu_t"), NULL, on_cut, GTK_STOCK_CUT),
+        GNOMEUIINFO_ITEM_STOCK(N_("_Copy"), NULL, on_copy, GTK_STOCK_COPY),
+        GNOMEUIINFO_ITEM_STOCK(N_("_Delete"), NULL, on_delete, GNOME_STOCK_TRASH),
         GNOMEUIINFO_SEPARATOR,
-        GNOMEUIINFO_ITEM_STOCK(N_("_Properties..."), NULL, on_properties, GNOME_STOCK_MENU_PROP),
+        GNOMEUIINFO_ITEM_NONE (N_("Rename"), NULL, on_rename),
+        GNOMEUIINFO_ITEM_STOCK(N_("Send files"), NULL, file_sendto, GNOME_STOCK_MAIL_SND),
+        GNOMEUIINFO_ITEM_FILENAME (N_("Open this _folder"), NULL, command_open_nautilus, PACKAGE_NAME G_DIR_SEPARATOR_S "nautilus.svg"),
+        GNOMEUIINFO_ITEM_FILENAME (N_("Open _terminal here"), NULL, command_open_terminal, PACKAGE_NAME G_DIR_SEPARATOR_S "terminal.svg"),
+        GNOMEUIINFO_SEPARATOR,
+        GNOMEUIINFO_ITEM_STOCK(N_("_Properties..."), NULL, on_properties, GTK_STOCK_PROPERTIES),
         GNOMEUIINFO_END
     };
 
     g_return_val_if_fail (GNOME_CMD_IS_FILE_LIST (fl), NULL);
-    GList *files = gnome_cmd_file_list_get_selected_files (fl);
+    GList *files = fl->get_selected_files();
     if (!files) return NULL;
 
     GnomeCmdFilePopmenu *menu = (GnomeCmdFilePopmenu *) gtk_type_new (gnome_cmd_file_popmenu_get_type ());
 
-    GnomeCmdFile *finfo = (GnomeCmdFile *) files->data;
+    GnomeCmdFile *f = (GnomeCmdFile *) files->data;
 
 
     // Fill the "Open with..." menu with applications
     gint i = -1;
     menu->priv->data_list = NULL;
 
-    vfs_apps = tmp = gnome_vfs_mime_get_all_applications (finfo->info->mime_type);
+    vfs_apps = tmp = gnome_vfs_mime_get_all_applications (f->info->mime_type);
     for (; vfs_apps && i < MAX_OPEN_WITH_APPS; vfs_apps = vfs_apps->next)
     {
         GnomeVFSMimeApplication *vfs_app = (GnomeVFSMimeApplication *) vfs_apps->data;
@@ -519,7 +528,7 @@ GtkWidget *gnome_cmd_file_popmenu_new (GnomeCmdFileList *fl)
     g_free ((gpointer) open_uiinfo[0].label);
 
     pos += 3;
-    if (gnome_cmd_file_is_executable (finfo) && g_list_length (files) == 1)
+    if (gnome_cmd_file_is_executable (f) && g_list_length (files) == 1)
         gnome_app_fill_menu (GTK_MENU_SHELL (menu), exec_uiinfo, NULL, FALSE, pos++);
 
     gnome_app_fill_menu (GTK_MENU_SHELL (menu), sep_uiinfo, NULL, FALSE, pos++);
@@ -596,7 +605,7 @@ GtkWidget *gnome_cmd_file_popmenu_new (GnomeCmdFileList *fl)
 }
 
 
-GtkType gnome_cmd_file_popmenu_get_type (void)
+GtkType gnome_cmd_file_popmenu_get_type ()
 {
     static GtkType dlg_type = 0;
 
