@@ -1,7 +1,7 @@
 /*
     GNOME Commander - A GNOME based file manager
     Copyright (C) 2001-2006 Marcus Bjurman
-    Copyright (C) 2007-2009 Piotr Eljasiak
+    Copyright (C) 2007-2010 Piotr Eljasiak
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -640,7 +640,7 @@ static void do_mime_exec_multiple (gpointer *args)
         if (dirs.size()==1)
             run_command_indir (cmd.c_str(), dirs.begin()->c_str(), gnome_cmd_app_get_requires_terminal (app));
         else
-            run_command (cmd.c_str(), gnome_cmd_app_get_requires_terminal (app));
+            run_command_indir (cmd.c_str(), NULL, gnome_cmd_app_get_requires_terminal (app));
 
         g_list_free (files);
     }
@@ -857,13 +857,6 @@ const gchar *create_nice_size_str (GnomeVFSFileSize size)
 }
 
 
-gchar *quote_if_needed (const gchar *in)
-{
-    //return strpbrk(in,";&$'\"?")==NULL ? g_strdup (in) : g_strdup_printf ("'%s'", in);
-    return g_shell_quote (in);
-}
-
-
 gchar *unquote_if_needed (const gchar *in)
 {
 
@@ -1270,7 +1263,7 @@ void edit_mimetypes (const gchar *mime_type, gboolean blocking)
     if (blocking)
         system (cmd);
     else
-        run_command (cmd, FALSE);
+        run_command (cmd);
     g_free (cmd);
 }
 
